@@ -1,5 +1,6 @@
 from nctoolkit.api import (
     open_data,
+    from_xarray,
     merge,
     cor_time,
     cor_space,
@@ -11,14 +12,16 @@ from nctoolkit.api import (
 from nctoolkit.cleanup import cleanup, clean_all, deep_clean, temp_check
 from nctoolkit.create_ensemble import create_ensemble
 from nctoolkit.session import session_files
-from nctoolkit.show import nc_variables, nc_years
+from nctoolkit.show import nc_variables, nc_years, nc_months, nc_times
 
 import re
 import subprocess
 
 # check version of cdo installed
 
-from nctoolkit.utils import validate_version
+from nctoolkit.utils import validate_version, cdo_version
+from nctoolkit.session import session_info
 
+session_info["cdo"] = cdo_version()
 
 validate_version()
