@@ -20,6 +20,7 @@ from nctoolkit.flatten import str_flatten
 from nctoolkit.runthis import run_cdo
 from nctoolkit.session import (
     nc_protected,
+    nc_protected_par,
     session_info,
     nc_safe,
     append_safe,
@@ -28,7 +29,7 @@ from nctoolkit.session import (
     remove_protected,
     append_tempdirs,
 )
-from nctoolkit.session import nc_safe_par, temp_dirs, nc_protected
+from nctoolkit.session import nc_safe_par, temp_dirs, nc_protected, temp_dirs_par
 from nctoolkit.show import (
     nc_variables,
     nc_years,
@@ -44,6 +45,7 @@ from nctoolkit.temp_file import temp_file
 import signal
 import time
 from contextlib import contextmanager
+import xarray as xr
 
 
 class TimeoutException(Exception):
@@ -142,7 +144,7 @@ def options(**kwargs):
         if key not in valid_keys:
             raise AttributeError(key + " is not a valid option")
 
-        if key == "parallel":
+        if key == "parallel" or key == "lazy" or key == "thread_safe":
             if type(kwargs[key]) is not bool:
                 raise TypeError(f"{key} should be boolean")
 
@@ -173,7 +175,7 @@ def options(**kwargs):
                         raise ValueError("precision supplied is not valid!")
                     session_info[key] = kwargs[key]
                 else:
-                    raise AttributeError(key + " is not valid session info!")
+                    raise ValueError(kwargs[key] + " is not valid session info!")
         else:
             session_info[key] = kwargs[key]
 
@@ -192,7 +194,6 @@ def options(**kwargs):
                     for ff in nc_protected:
                         nc_protected_par.append(ff)
                         nc_protected.remove(ff)
-                        nc_protected.remove(ff)
 
             if (kwargs[key] == False) and key == "parallel":
 
@@ -239,7 +240,6 @@ if config_file is not None:
     file1 = open(config_file, "r")
     Lines = file1.readlines()
 
-    # pattern = re.compile("[a-z]+\:[0-9A-Za-z]+")
     count = 0
     # Strips the newline character
     for line in Lines:
@@ -368,6 +368,27 @@ def file_size(file_path):
         return file_info.st_size
 
 
+def from_xarray(ds):
+
+    """
+    Convert an xarray dataset to an nctoolkit dataset
+    This will first save the xarray dataset as a temporary netCDF file.
+
+    Parameters
+    ---------------
+    ds : xarray dataset
+    ---------------
+    from_xarray : nctoolkit.DataSet
+    """
+    ff = temp_file(".nc")
+    ds.to_netcdf(ff)
+
+    append_safe(ff)
+
+    d = DataSet(ff)
+    return d
+
+
 def open_data(x=[], checks=False, **kwargs):
     """
     Read netCDF data as a DataSet object
@@ -420,6 +441,9 @@ def open_data(x=[], checks=False, **kwargs):
     ftp_details = None
     wait = None
     file_stop = None
+
+    source = "file"
+
     for key in kwargs:
         if key == "thredds":
             thredds = kwargs[key]
@@ -429,6 +453,8 @@ def open_data(x=[], checks=False, **kwargs):
             wait = kwargs[key]
         if key == "file_stop":
             file_stop = kwargs[key]
+        if key == "source":
+            source = kwargs[key]
 
     # make sure data has been supplied
     if x is None:
@@ -453,7 +479,8 @@ def open_data(x=[], checks=False, **kwargs):
         stop_time = min(file_stop, stop_time)
 
     if type(x) is str:
-        if os.path.exists(x) is False:
+        ##if os.path.exists(x) is False:
+        if source != "file" or os.path.exists(x) is False:
 
             if is_url(x):
 
@@ -511,18 +538,9 @@ def open_data(x=[], checks=False, **kwargs):
                                 )
                                 if "Open failed" in str(out.stderr):
                                     raise ValueError(f"{x} is not compatible with CDO!")
-                        #else:
-                        #    out = subprocess.run(
-                        #        "cdo sinfo " + x,
-                        #        shell=True,
-                        #        stdout=subprocess.PIPE,
-                        #        stderr=subprocess.PIPE,
-                        #    )
-                        #    if "Open failed" in str(out.stderr):
-                        #        raise ValueError(f"{x} is not compatible with CDO!")
 
             else:
-                raise ValueError("Data set " + x + " does not exist!")
+                raise FileNotFoundError("Data set " + x + " does not exist!")
 
         if checks:
             out = subprocess.run(
@@ -555,48 +573,77 @@ def open_data(x=[], checks=False, **kwargs):
         if len(x) < orig_size:
             warnings.warn(message="Duplicates in data set have been removed!")
 
-    # if type(x) is list:
-    #    if len(x) == 1:
-    #        x = x[0]
-
     if type(x) is list:
+        if source == "url":
+
+            for ff in x:
+
+                if is_url(ff) == False:
+                    raise ValueError(f"{x} is not a url")
+
+            new_files = []
+            for ff in x:
+
+                if is_url(ff):
+
+                    if thredds is False:
+                        new_x = temp_file(".nc")
+                        print(f"Downloading {ff}")
+                        print("\033[A                             \033[A")
+
+                        if ftp_details is not None and x.startswith("ftp"):
+                            user = ftp_details["user"]
+                            password = ftp_details["password"]
+                            ff = ff.replace("ftp://", f"ftp://{user}:{password}@")
+
+                        start = time.time()
+
+                        i = 0
+
+                        search = True
+                        while search:
+                            if os.path.exists(new_x) == False:
+                                try:
+                                    # work out if there is a time limit for individual files
+                                    if stop_time != 10000000000000000000000000000000000:
+                                        with time_limit(stop_time):
+                                            urllib.request.urlretrieve(ff, new_x)
+                                    else:
+                                        urllib.request.urlretrieve(ff, new_x)
+                                except:
+                                    nothing = "x"
+                            search += 1
+                            if os.path.exists(new_x):
+                                break
+
+                            if wait is None:
+                                if search == 3:
+                                    break
+                            else:
+                                end = time.time()
+                                if (end - start) > wait:
+                                    break
+                        if os.path.exists(new_x) == False:
+                            raise ValueError(f"Could not download {x}")
+
+                        new_files.append(new_x)
+                x = new_files
+
+    if type(x) is list and source != "url":
         if thredds is False:
 
+            for ff in x:
+                if os.path.exists(ff) is False:
+                    raise FileNotFoundError(f"{ff} does not exist!")
+            for ff in x:
+                append_safe(ff)
+                append_protected(ff)
+
             if len(x) > 1:
                 for ff in x:
 
                     if os.path.exists(ff) is False:
-                        raise ValueError("Data set " + ff + " does not exist!")
-                    else:
-                        if checks:
-                            out = subprocess.run(
-                                "cdo sinfo " + ff,
-                                shell=True,
-                                stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE,
-                            )
-                            if "Open failed" in out.stderr.decode("utf-8"):
-                                mes = (
-                                    out.stderr.decode("utf-8")
-                                    .replace("cdo    sinfo: ", "")
-                                    .replace("<\n", "")
-                                    .replace("\n", "")
-                                )
-                                mes = re.sub(" +", " ", mes)
-                                raise ValueError(mes)
-                        append_safe(ff)
-                        append_protected(ff)
-        else:
-            out = subprocess.run(
-                "cdo sinfo " + x[0],
-                shell=True,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
-            )
-            if "Open failed" in str(out.stderr):
-                raise ValueError(
-                    "First file in the ensemble is not compatible with CDO!"
-                )
+                        raise FileNotFoundError("Data set " + ff + " does not exist!")
 
     # if there is only one file in the list, change it to a single file
     if type(x) is list:
@@ -605,6 +652,20 @@ def open_data(x=[], checks=False, **kwargs):
 
     d = DataSet(x)
     d._thredds = thredds
+
+    if len(d) == 1:
+        df = d.contents.reset_index(drop=True).query("'I' in data_type")
+        if len(df) > 0:
+            check = ",".join(list(df.variable))
+            if "," in check:
+                warnings.warn(
+                    message=f"The variable(s) {check} have integer data type. Consider setting data type to float 'F64' or 'F32' using set_precision."
+                )
+            else:
+                warnings.warn(
+                    message=f"The variable {check} has integer data type. Consider setting data type to float 'F64' or 'F32' using set_precision."
+                )
+
     return d
 
 
@@ -651,7 +712,7 @@ def open_thredds(x=None, wait=None, checks=False):
         )
         session_info["cores"] = 1
 
-    return open_data(x=x, thredds=True, wait=wait, checks=checks)
+    return open_data(x=x, thredds=True, wait=wait, checks=checks, source="thredds")
 
 
 def open_url(x=None, ftp_details=None, wait=None, file_stop=None):
@@ -710,7 +771,9 @@ def open_url(x=None, ftp_details=None, wait=None, file_stop=None):
     else:
         new_dict = None
 
-    return open_data(x=x, ftp_details=new_dict, wait=wait, file_stop=file_stop)
+    return open_data(
+        x=x, ftp_details=new_dict, wait=wait, file_stop=file_stop, source="url"
+    )
 
 
 def merge(*datasets, match=["day", "year", "month"]):
@@ -791,7 +854,7 @@ def cor_time(x=None, y=None):
     else:
         command = "cdo -timcor " + a.current[0] + " " + b.current[0] + " " + target
 
-    target = run_cdo(command, target=target)
+    target = run_cdo(command, target=target, precision=x._precision)
 
     data = open_data(target)
 
@@ -852,7 +915,7 @@ def cor_space(x=None, y=None):
     else:
         command = "cdo -fldcor " + a.current[0] + " " + b.current[0] + " " + target
 
-    target = run_cdo(command, target=target)
+    target = run_cdo(command, target=target, precision=x._precision)
 
     data = open_data(target)
 
@@ -889,12 +952,13 @@ class DataSet(object):
         self._zip = False
         self._format = None
 
+        self._precision = "default"
+
         self._grid = None
         self._weights = None
         # track number of held over commands
         self._ncommands = 0
 
-
     def __getitem__(self, index):
 
         return self.current[index]
@@ -1057,6 +1121,187 @@ class DataSet(object):
 
         return all_years
 
+    @property
+    def contents(self):
+        """
+        Detailed list of variables contained in a dataset.
+        This will only display the variables in the first file of an ensemble.
+        """
+
+        list_contents = []
+
+        use_names = True
+
+        for ff in self:
+            if "nctoolkit" in ff:
+                use_names = False
+
+        for ff in self:
+
+            dataset = Dataset(ff)
+
+            out = subprocess.run(
+                "cdo sinfon " + ff,
+                shell=True,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+            )
+            if "Unsupported file structure" in str(out.stderr):
+                for ff in self:
+                    remove_safe(ff)
+                    remove_protected(ff)
+                raise ValueError("Unsupported file structure")
+            if "expandWildCards" in out.stderr.decode("utf-8"):
+                for ff in self:
+                    remove_safe(ff)
+                    remove_protected(ff)
+                raise ValueError(out.stderr.decode("utf-8"))
+            out = out.stdout.decode("utf-8")
+            out = out.split("\n")
+            out_inc = ["Grid coordinates :" in ff for ff in out]
+            var_det = []
+            i = 1
+            while True:
+                if out_inc[i]:
+                    break
+                i += 1
+                var_det.append(out[i - 1])
+
+            def split_var(x):
+                x = x.replace(":", "").split(" ")
+                x = [x for x in x if len(x) > 0]
+                new_x = []
+                include = False
+                for y in x[1:]:
+                    if y.isnumeric():
+                        include = True
+
+                    if include:
+                        new_x.append(y)
+                return new_x
+
+            def fix_head(x):
+                x = (
+                    x.replace(":", "")
+                    .replace("  ", " ")
+                    .replace("Parameter name", "variable")
+                )
+                x = x[x.find("Level") :].replace("  ", " ")
+                return x.split(" ")
+
+            def fix_type(x):
+                position = [m.start() for m in re.finditer(":", x)][-1]
+
+                return (
+                    x[: (position - 4)]
+                    + x[(position - 4) : position].replace(" ", "")
+                    + " "
+                    + x[(position):]
+                )
+
+            var_det[0] = fix_head(var_det[0])
+            for ii in range(1, len(var_det)):
+                var_det[ii] = fix_type(var_det[ii])
+                var_det[ii] = split_var(var_det[ii])
+
+            df = pd.DataFrame.from_records(var_det[1:], columns=var_det[0])
+            df = df.loc[:, ["Levels", "Points", "variable", "Dtype"]]
+            df = df.rename(
+                columns={"Levels": "nlevels", "Points": "npoints", "Dtype": "data_type"}
+            )
+
+            longs = None
+            units = None
+            longs = []
+
+            cdo_result = list(df.variable)
+
+            for x in cdo_result:
+                try:
+                    longs.append(dataset.variables[x].long_name)
+                except:
+                    longs.append(None)
+            units = []
+            for x in cdo_result:
+                try:
+                    units.append(dataset.variables[x].units)
+                except:
+                    units.append(None)
+
+            df = pd.DataFrame({"variable": cdo_result}).merge(df)
+
+            if longs is not None:
+                df["long_name"] = longs
+            if units is not None:
+                df["unit"] = units
+
+            df = df.assign(nlevels=lambda x: x.nlevels.astype("int")).assign(
+                npoints=lambda x: x.npoints.astype("int")
+            )
+
+            times = []
+
+            ds = Dataset(ff)
+            time_name = [x for x in ds.variables if "time" in x]
+
+            # get time name
+
+            if len(time_name) > 0:
+                time_name = time_name[0]
+            else:
+                time_name = "time"
+
+            for vv in cdo_result:
+                done = False
+                try:
+                    ds_times = ds[vv][time_name].values
+                except:
+                    times.append(None)
+                    done = True
+
+                if done is False:
+                    try:
+                        n_times = len(ds_times)
+                    except:
+                        n_times = 1
+                    times.append(n_times)
+
+            df["ntimes"] = times
+
+            df = df.loc[
+                :,
+                [
+                    "variable",
+                    "ntimes",
+                    "npoints",
+                    "nlevels",
+                    "long_name",
+                    "unit",
+                    "data_type",
+                ],
+            ]
+            list_contents.append(df.assign(file=ff))
+
+        if len(list_contents) == 1:
+            return list_contents[0].drop(columns="file")
+
+        if use_names == False:
+            new_df = []
+            i = 1
+            for x in list_contents:
+                new_df.append(x.assign(file=f"file {i}"))
+                i += 1
+
+            new_df = pd.concat(new_df)
+
+            if len(set(new_df.file)) == 1:
+                return new_df.drop(columns="file")
+            else:
+                new_df = new_df.set_index("file")
+                return new_df
+        else:
+            return pd.concat(list_contents).set_index("file")
+
     @property
     def variables_detailed(self):
         """
@@ -1088,10 +1333,20 @@ class DataSet(object):
 
         longs = None
         units = None
-        if "long_name" in str(dataset.variables[cdo_result[0]]):
-            longs = [dataset.variables[x].long_name for x in cdo_result]
-        if "units" in str(dataset.variables[cdo_result[0]]):
-            units = [dataset.variables[x].units for x in cdo_result]
+        longs = []
+        for x in cdo_result:
+            try:
+                longs.append(dataset.variables[x].long_name)
+            except:
+                longs.append(None)
+        # longs = [dataset.variables[x].long_name for x in cdo_result]
+        units = []
+        for x in cdo_result:
+            try:
+                units.append(dataset.variables[x].units)
+            except:
+                units.append(None)
+        ##units = [dataset.variables[x].units for x in cdo_result]
 
         if longs is None and units is None:
             return cdo_result
@@ -1118,9 +1373,10 @@ class DataSet(object):
         var_det = [
             ff.replace("Parameter name", "variable").split(" ") for ff in var_det
         ]
-        sales = var_det[1:]
+        df_vars = var_det[1:]
         labels = var_det[0]
-        df = pd.DataFrame.from_records(sales, columns=labels)
+
+        df = pd.DataFrame.from_records(df_vars, columns=labels)
         df = df.loc[:, ["Levels", "Points", "variable"]]
         df = df.rename(columns={"Levels": "levels", "Points": "points"})
 
@@ -1275,6 +1531,12 @@ class DataSet(object):
     from nctoolkit.fldstat import spatial_sum
     from nctoolkit.fldstat import spatial_percentile
 
+    from nctoolkit.fldstat import box_mean
+    from nctoolkit.fldstat import box_max
+    from nctoolkit.fldstat import box_min
+    from nctoolkit.fldstat import box_sum
+    from nctoolkit.fldstat import box_range
+
     from nctoolkit.format import format
 
     from nctoolkit.inttime import time_interp
@@ -1283,7 +1545,6 @@ class DataSet(object):
     from nctoolkit.masking import mask_box
 
     from nctoolkit.mergers import collect
-    from nctoolkit.mergers import merge_time
     from nctoolkit.mergers import merge
 
     from nctoolkit.meridonials import meridonial_mean
@@ -1305,7 +1566,6 @@ class DataSet(object):
 
     from nctoolkit.regrid import regrid
 
-
     from nctoolkit.rename import rename
 
     from nctoolkit.resample import resample_grid
@@ -1324,6 +1584,7 @@ class DataSet(object):
     from nctoolkit.setters import set_missing
     from nctoolkit.setters import set_units
     from nctoolkit.setters import set_longnames
+    from nctoolkit.setters import set_precision
 
     from nctoolkit.shift import shift
 
@@ -1343,6 +1604,8 @@ class DataSet(object):
     from nctoolkit.temporal_stat import tstdev
     from nctoolkit.temporal_stat import tsum
     from nctoolkit.temporal_stat import tcumsum
+    from nctoolkit.temporal_stat import na_count
+    from nctoolkit.temporal_stat import na_frac
 
     from nctoolkit.thresholds import first_above
     from nctoolkit.thresholds import first_below
@@ -1361,14 +1624,14 @@ class DataSet(object):
     from nctoolkit.verticals import vertical_max
     from nctoolkit.verticals import vertical_range
     from nctoolkit.verticals import vertical_sum
+    from nctoolkit.verticals import vertical_integration
     from nctoolkit.verticals import vertical_cumsum
-    from nctoolkit.verticals import surface
+    from nctoolkit.verticals import top
     from nctoolkit.verticals import vertical_interp
     from nctoolkit.verticals import bottom
     from nctoolkit.verticals import bottom_mask
     from nctoolkit.verticals import invert_levels
 
-
     from nctoolkit.zip import zip
 
     from nctoolkit.zonals import zonal_mean
@@ -1376,6 +1639,6 @@ class DataSet(object):
     from nctoolkit.zonals import zonal_max
     from nctoolkit.zonals import zonal_range
 
-    # deprecated stuff
-
-    from nctoolkit.compare import compare_all
+    # Deprecated methods
+    from nctoolkit.deprecated import merge_time
+    from nctoolkit.deprecated import surface
