@@ -27,6 +27,7 @@ import numpy as np
 from nctoolkit.flatten import str_flatten
 from nctoolkit.runthis import run_this
 from nctoolkit.session import session_info
+from nctoolkit.utils import cdo_version, version_below
 
 
 def split_equation(mystr):
@@ -101,7 +102,7 @@ funs = [
     "tan",
     "int",
     "float",
-    "log10"
+    "log10",
 ]
 
 translation = dict()
@@ -112,6 +113,10 @@ for ff in funs:
 translation["int"] = "int"
 translation["float"] = "float"
 
+# if version_below(session_info["cdo"], "2.0.0") is False:
+# translation["thickness"] = "cdeltaz"
+
+
 translation["arcsin"] = "asin"
 translation["arccos"] = "acos"
 translation["arctan"] = "atan"
@@ -189,6 +194,9 @@ def assign(self, drop=False, **kwargs):
     Operations are carried out in the order give. So if a new variable is created in the first argument,
     it can then be used in following arguments.
 
+
+
+
     """
     frame = inspect.currentframe()
 
@@ -249,9 +257,9 @@ def assign(self, drop=False, **kwargs):
             for ss in start:
                 if ".assign(" in ss:
                     ind = i
-                i+=1
+                i += 1
 
-            all_str = start[ind:len(start)]
+            all_str = start[ind : len(start)]
             all_str = "".join(all_str)
             start = all_str.replace("'", "").replace("\\n", "").replace("  ", " ")
         else:
@@ -259,12 +267,13 @@ def assign(self, drop=False, **kwargs):
             try:
                 all_str = list(inspect.getsourcelines(start))[0]
                 all_str = "".join(all_str)
-                start = all_str.replace("'", "").replace("\\n", "")[1:-1].replace("  ", " ")
+                start = (
+                    all_str.replace("'", "").replace("\\n", "")[1:-1].replace("  ", " ")
+                )
 
             except:
                 start = dill.source.getsource(start).replace("\n", "").strip()
 
-
     # we now need to figure out if what we have is one line
 
     if ";" in start:
@@ -280,7 +289,7 @@ def assign(self, drop=False, **kwargs):
 
     start = start.strip()
     start = start[start.find("(") + 1 : -1]
-    start = re.sub("\s\s+" , " ", start)
+    start = re.sub("\s\s+", " ", start)
 
     pattern1 = re.compile("drop\s*=\s*(True|False)")
     y = pattern1.search(start)
@@ -447,18 +456,32 @@ def assign(self, drop=False, **kwargs):
                                 start_parens = find_parens3(start)
                                 x_term = (
                                     x
-                                    + start[y.span()[1] : start_parens[y.span()[1] - 1] + 1]
+                                    + start[
+                                        y.span()[1] : start_parens[y.span()[1] - 1] + 1
+                                    ]
                                 )
                                 start = start.replace(x_term, x_term.replace(" ", ""))
                                 if old_start != start:
                                     break
-                        n+=1
+                        n += 1
 
                         if n > n_limit:
                             fix = False
 
         error_message = None
 
+        if "is False" in start:
+            start = start.replace(" is False", " < 1")
+
+        if "is True" in start:
+            start = start.replace(" is True", " > 0")
+
+        if " = = True" in start:
+            start = start.replace(" = = True", " > 0")
+
+        if " = = False" in start:
+            start = start.replace(" = = False", " < 1")
+
         for x in start.split(" "):
             if "(" in x:
                 x_fun = x.split("(")[0]
@@ -624,8 +647,15 @@ def assign(self, drop=False, **kwargs):
                             raise ValueError(f"{term} does not evaluate to a numeric!")
 
                         if is_number(str(new_term)) == False:
-                            error_message = f"{term} does not evaluate to a numeric!"
-                            raise ValueError(f"{term} does not evaluate to a numeric!")
+                            if type(new_term) is bool:
+                                new_term = float(new_term)
+                            else:
+                                error_message = (
+                                    f"{term} does not evaluate to a numeric!"
+                                )
+                                raise ValueError(
+                                    f"{term} does not evaluate to a numeric!"
+                                )
 
                         new_start = ""
                         for y in start.split(" "):
@@ -703,7 +733,7 @@ def assign(self, drop=False, **kwargs):
 
         if "." in term:
             pattern1 = re.compile(r"[a-zA-Z]")
-            if len(pattern1.findall(term)) > 0:
+            if len(pattern1.findall(term)) > 0 and "^" not in term:
                 if term.split(".")[0] in frame.f_back.f_locals:
                     try:
                         new_term = eval(term, globals(), frame.f_back.f_locals)
