@@ -7,13 +7,100 @@ from nctoolkit.temp_file import temp_file
 from nctoolkit.session import nc_safe, remove_safe, append_safe
 
 
-def cdo_version():
-    cdo_check = subprocess.run(
-        "cdo --version", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-    )
-    cdo_check = str(cdo_check.stderr).replace("\\n", "")
-    cdo_check = cdo_check.replace("b'", "").strip()
-    return cdo_check.split("(")[0].strip().split(" ")[-1]
+def boxstat(self, stat="mean", x=1, y=1):
+    """Method to calculate the spatial stat from a dataset"""
+
+    if len(self) == 0:
+        raise ValueError("Failure due to empty dataset!")
+
+    cdo_command = f"cdo -gridbox{stat},{x},{y}"
+
+    run_this(cdo_command, self, output="ensemble")
+
+
+def box_mean(self, x=1, y=1):
+    """
+    Calculate the grid box mean for all variables
+    This is performed for each time step.
+
+    Parameters
+    -------------
+    x: int
+        Number of boxes in the x, e.g. east-west direction
+    y: int or float
+        Number of boxes in the y, e.g. north-south direction
+
+    """
+
+    boxstat(self, stat="mean", x=x, y=y)
+
+
+def box_max(self, x=1, y=1):
+    """
+    Calculate the grid box max for all variables
+    This is performed for each time step.
+
+    Parameters
+    -------------
+    x: int
+        Number of boxes in the x, e.g. east-west direction
+    y: int or float
+        Number of boxes in the y, e.g. north-south direction
+
+    """
+
+    boxstat(self, stat="max", x=x, y=y)
+
+
+def box_min(self, x=1, y=1):
+    """
+    Calculate the grid box min for all variables
+    This is performed for each time step.
+
+    Parameters
+    -------------
+    x: int
+        Number of boxes in the x, e.g. east-west direction
+    y: int or float
+        Number of boxes in the y, e.g. north-south direction
+
+    """
+
+    boxstat(self, stat="min", x=x, y=y)
+
+
+def box_sum(self, x=1, y=1):
+    """
+    Calculate the grid box sum for all variables
+    This is performed for each time step.
+
+    Parameters
+    -------------
+    x: int
+        Number of boxes in the x, e.g. east-west direction
+    y: int or float
+        Number of boxes in the y, e.g. north-south direction
+
+    """
+
+    boxstat(self, stat="sum", x=x, y=y)
+
+
+def box_range(self, x=1, y=1):
+    """
+    Calculate the grid box range for all variables
+    This is performed for each time step.
+
+    Parameters
+    -------------
+    x: int
+        Number of boxes in the x, e.g. east-west direction
+    y: int or float
+        Number of boxes in the y, e.g. north-south direction
+
+    """
+
+    boxstat(self, stat="range", x=x, y=y)
 
 
 def fldstat(self, stat="mean"):
@@ -22,7 +109,6 @@ def fldstat(self, stat="mean"):
     if len(self) == 0:
         raise ValueError("Failure due to empty dataset!")
 
-
     cdo_command = f"cdo -fld{stat}"
 
     run_this(cdo_command, self, output="ensemble")
@@ -149,7 +235,7 @@ def spatial_sum(self, by_area=False):
 
         cdo_command = f"cdo -fldsum -mul {ff} -gridarea {ff} {target}"
         cdo_command = tidy_command(cdo_command)
-        target = run_cdo(cdo_command, target=target)
+        target = run_cdo(cdo_command, target=target, precision=self._precision)
         new_files.append(target)
         new_commands.append(cdo_command)
 
