@@ -6,34 +6,89 @@ from datetime import datetime
 
 from nctoolkit.runthis import run_this
 from nctoolkit.session import session_info
+from nctoolkit.show import nc_variables, nc_times
+from nctoolkit.utils import cdo_version, version_below, version_above
 
+def below(x,y):
+    x = x.split(".")
+    x = int(x[0])* 1000 +  int(x[1]) * 100+  int(x[2])
 
-def cdo_version():
-    """Function to find cdo version"""
-    cdo_check = subprocess.run(
-        "cdo --version", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-    )
-    cdo_check = str(cdo_check.stderr).replace("\\n", "")
-    cdo_check = cdo_check.replace("b'", "").strip()
-    return cdo_check.split("(")[0].strip().split(" ")[-1]
+    y = y.split(".")
+    y = int(y[0])* 1000 +  int(y[1]) * 100+  int(y[2])
 
+    return x < y
 
-def merge(self, match=["year", "month", "day"]):
+def merge(self, join="variables", match=["year", "month", "day"]):
     """
     Merge a multi-file ensemble into a single file
-    Merging will occur based on the time steps in the first file.
-    This will only be effective if you want to merge files with the same times,
-    but with different variables.
+    2 methods are available. 1) merging files with different variables, but the same time steps.
+    2) merging files with the same variables, with different times.
 
     Parameters
     -------------
+    join: str
+        This defines the type of merging to carry out. "variables": this will merge by variable, so that an ensemble
+        with different variables, but the same number of time steps is merged to a single file.
+        "time": this will merge files with the same variables, but different times to a single file, into a single file
+        with ordered times.  join defaults to "variables", and uses partial matches, so "var" will give variable based merging.
+
     match: list, str
-        a list or str stating what must match in the netCDF files.
+        Optional argument when join = 'variables'. A list or str stating what must match in the netCDF files.
         Defaults to year/month/day. This list must be some combination of
         year/month/day. An error will be thrown if the elements of time in match
         do not match across all netCDF files. The only exception is if there is a
         single date file in the ensemble.
     """
+    if type(join) is not str:
+        raise TypeError("join supplied is not a str")
+
+    join_valid = False
+
+    if join.startswith("var"):
+        join_valid = True
+
+    if join.startswith("time"):
+        self.run()
+        if version_below(cdo_version(), "1.9.9"):
+            var_list = []
+            var_com = []
+
+            for ff in self:
+                var_list += nc_variables(ff)
+                var_com.append(nc_variables(ff))
+            new_list = []
+
+            for var in set(var_list):
+                if len(var_com) == len([x for x in var_com if var in x]):
+                    new_list.append(var)
+
+            self.select(variables=new_list)
+            self.run()
+
+            removed = ",".join([x for x in set(var_list) if x not in new_list])
+            if len([x for x in set(var_list) if x not in new_list]) > 0:
+                warnings.warn(
+                    f"The following variables are not in all files, so were ignored when merging: {removed}"
+                )
+                self.select(variables=new_list)
+                self.run()
+
+        if len(self) == 1:
+            warnings.warn(
+                message="There is only file in the dataset. No need to merge!"
+            )
+            return None
+
+        cdo_command = "cdo --sortname -mergetime"
+
+        run_this(cdo_command, self, output="one")
+
+        if session_info["lazy"]:
+            self._merged = True
+        return None
+
+    if join_valid == False:
+        raise ValueError("join supplied is not valid")
 
     # basic checks on match criteria
     if type(match) is str:
@@ -71,9 +126,12 @@ def merge(self, match=["year", "month", "day"]):
             shell=True,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
-        ).stdout
-        cdo_result = str(cdo_result).replace("b'", "").strip()
-        ntime = int(cdo_result.split("\\")[0])
+        )
+        cdo_result =  cdo_result.stdout.decode("utf-8")
+        #.stdout
+        #''cdo_result = str(cdo_result).replace("b'", "").strip()
+        cdo_result = str(cdo_result)
+        ntime = int(cdo_result.split("\n")[0])
         all_times.append(ntime)
     if len(set(all_times)) > 1:
         warnings.warn(
@@ -100,15 +158,15 @@ def merge(self, match=["year", "month", "day"]):
     # check the file times are compatible
     all_times = []
     for ff in self:
-        cdo_result = subprocess.run(
-            f"cdo showtimestamp {ff}",
-            shell=True,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-        ).stdout
-        cdo_result = str(cdo_result).replace("b'", "").strip()
-        cdo_result = cdo_result.split()
-        cdo_result = pd.Series((v for v in cdo_result))
+        cdo_result = nc_times(ff)
+        #    f"cdo showtimestamp {ff}",
+        #    shell=True,
+        #    stdout=subprocess.PIPE,
+        #    stderr=subprocess.PIPE,
+        #).stdout
+        #cdo_result = str(cdo_result).replace("b'", "").strip()
+        #cdo_result = cdo_result.split()
+        #cdo_result = pd.Series((v for v in cdo_result))
         all_times.append(cdo_result)
 
     for i in range(1, len(all_times)):
@@ -124,9 +182,9 @@ def merge(self, match=["year", "month", "day"]):
     all_df = []
     if len(all_times) > 1:
         for i in range(0, len(all_times)):
-            month = [datetime.strptime(v[0:10], "%Y-%m-%d").month for v in all_times[i]]
-            year = [datetime.strptime(v[0:10], "%Y-%m-%d").year for v in all_times[i]]
-            day = [datetime.strptime(v[0:10], "%Y-%m-%d").day for v in all_times[i]]
+            month = [v.month for v in all_times[i]]
+            year = [v.year for v in all_times[i]]
+            day = [v.day for v in all_times[i]]
             i_data = pd.DataFrame({"year": year, "month": month, "day": day})
             i_data = i_data.loc[:, match]
             all_df.append(i_data)
@@ -143,29 +201,6 @@ def merge(self, match=["year", "month", "day"]):
         self._merged = True
 
 
-def merge_time(self):
-    """
-    Time-based merging of a multi-file ensemble into a single file
-    This method is ideal if you have the same data split over multiple
-    files covering different data sets.
-    """
-
-    self.run()
-
-    if len(self) == 1:
-        warnings.warn(message="There is only file in the dataset. No need to merge!")
-        return None
-
-    cdo_command = "cdo --sortname -mergetime"
-
-    run_this(cdo_command, self, output="one")
-
-    if session_info["lazy"]:
-        self._merged = True
-
-
-
-
 def collect(self):
     """
     Collect a dataset that has been split using distribute
@@ -185,4 +220,3 @@ def collect(self):
         self._merged = True
 
     self.run()
-
