@@ -12,7 +12,7 @@ from nctoolkit.session import nc_safe, append_safe, remove_safe, get_safe
 from nctoolkit.temp_file import temp_file
 
 
-def regrid(self, grid=None, method="bil", recycle = False):
+def regrid(self, grid=None, method="bil", recycle=False):
     """
     Regrid a dataset to a target grid
 
@@ -43,11 +43,9 @@ def regrid(self, grid=None, method="bil", recycle = False):
             weights_nc = grid._weights
             cdo_command = f"cdo -remap,{target_grid},{weights_nc}"
             run_this(cdo_command, self, output="ensemble")
-            print("recycling")
 
             return None
 
-
     del_grid = None
     if grid is None:
         raise ValueError("No grid was supplied")
@@ -57,6 +55,8 @@ def regrid(self, grid=None, method="bil", recycle = False):
     # find the grid type
     if isinstance(grid, pd.DataFrame):
         grid_type = "df"
+        if len(grid) == 0:
+            raise ValueError("You have supplied an empty data frame as a grid!")
 
     if type(grid) is str:
         if os.path.exists(grid) is False:
@@ -88,7 +88,6 @@ def regrid(self, grid=None, method="bil", recycle = False):
     if len(self) > 1 and recycle:
         raise ValueError("You cannot recycle multi-file datasets")
 
-
     for ff in self:
         cdo_result = subprocess.run(
             f"cdo griddes {ff}",
@@ -122,15 +121,19 @@ def regrid(self, grid=None, method="bil", recycle = False):
             grid_split[key], suppress_messages=True, thredds=self._thredds
         )
 
-        weights_nc = temp_file("nc")
+        if tracker._precision != "default":
+            tracker.set_precision(self._precision)
 
+        weights_nc = temp_file("nc")
 
         cdo_command = (
             f"cdo -gen{method},{target_grid} {tracker.current[0]} {weights_nc}"
         )
 
         try:
-            weights_nc = run_cdo(cdo_command, target=weights_nc)
+            weights_nc = run_cdo(
+                cdo_command, target=weights_nc, precision=self._precision
+            )
         except Exception as e:
             del tracker
             remove_safe(weights_nc)
