@@ -17,6 +17,8 @@ from nctoolkit.session import (
 )
 from nctoolkit.temp_file import temp_file
 
+from nctoolkit.show import nc_variables
+
 from nctoolkit.utils import cdo_version
 
 
@@ -148,7 +150,19 @@ def run_nco(command, target, out_file=None, overwrite=False):
     return target
 
 
-def run_cdo(command, target, out_file=None, overwrite=False):
+def run_cdo(command = None, target = None, out_file=None, overwrite=False, precision = None):
+
+
+    if type(precision) is not str:
+        raise ValueError("Precision must be str")
+    
+    if precision not in ["I8", "I16", "I32", "F32", "F64", "default"]:
+        raise ValueError(f"Precision - {precision} - is not valid")
+        
+    if precision in ["I8", "I16", "I32", "F32", "F64"]:
+        if " -b " not in command:
+            command = command.replace("cdo ", f"cdo -b {precision} ")
+
 
     # make sure the output file does not exist
 
@@ -186,9 +200,18 @@ def run_cdo(command, target, out_file=None, overwrite=False):
     out.wait()
     result, ignore = out.communicate()
 
-    # this will potentially fail because of floating point precision. A quick fix to see if that is the case....
+    # If it is a generic grid, it's better to not throw the CDO error which might be confusing.
+    if "generic" in result.decode("utf-8").lower():
+        if "unsupported" in result.decode("utf-8").lower():
+            raise ValueError(
+                "Dataset contains generic grids. Please fix to lonlat or curvilinear"
+            )
 
-    if "Use the CDO option -b F32" in (result.decode("utf-8")):
+    # this will potentially fail because of floating point precision. A quick fix to see if that is the case....
+    if "Use the CDO option -b F32" in (
+        result.decode("utf-8")
+    ) or "not represent" in result.decode("utf-8"):
+        print("Switching to 32 bit precision!")
         command_chunks = command.split(" ")
 
         i = 0
@@ -198,11 +221,20 @@ def run_cdo(command, target, out_file=None, overwrite=False):
             if "-b" == cc.strip():
                 change = i
         if change is not None:
-            command_chunks[change] = "F32"
+            command_chunks[change] = "32"
             command = str_flatten(command_chunks, " ")
         else:
-            command = command.replace("cdo ", "cdo -b F32 ")
-        command
+            command = command.replace("cdo ", "cdo -b 32 ")
+
+        if os.path.exists(target):
+            if out_file is None:
+                os.remove(target)
+                remove_safe(target)
+
+        new_target = temp_file("nc")
+        append_safe(new_target)
+        command = command.replace(target, new_target)
+        target = new_target
 
         out = subprocess.Popen(
             command,
@@ -437,7 +469,7 @@ def run_this(os_command, self, output="one", out_file=None):
     if len(self) == 1:
         output = "ensemble"
 
-    if (len(self._hold_history) != len(self.history)):
+    if len(self._hold_history) != len(self.history):
         if self._merged == False and len(self.history) > 0 and output == "one":
 
             if cdo_version() in ["9.9.9"]:
@@ -446,7 +478,6 @@ def run_this(os_command, self, output="one", out_file=None):
                 if "apply," not in the_command:
                     self.history[-1] = f'-apply,"{the_command}"'
 
-
     if self._execute is False:
         if len(self._hold_history) == len(self.history):
             self.history.append(os_command)
@@ -530,10 +561,11 @@ def run_this(os_command, self, output="one", out_file=None):
                 new_history.append(ff_command)
 
                 if cores > 1:
-                    temp = pool.apply_async(run_cdo, [ff_command, target, out_file])
+                    temp = pool.apply_async(run_cdo, [ff_command, target, out_file,  False, self._precision])
                     results[ff] = temp
                 else:
-                    target = run_cdo(ff_command, target, out_file)
+
+                    target = run_cdo(ff_command, target, out_file, precision = self._precision)
                     target_list.append(target)
 
             if cores > 1:
@@ -567,9 +599,6 @@ def run_this(os_command, self, output="one", out_file=None):
 
             new_history = copy.deepcopy(self._hold_history)
 
-            file_list = self.current
-
-
             if len(self.history) > len(self._hold_history):
                 os_command = f'{os_command} {self.history[-1].replace("cdo ", " ")}'
                 os_command = os_command.replace("  ", " ")
@@ -597,7 +626,6 @@ def run_this(os_command, self, output="one", out_file=None):
                 os_command + " [ " + str_flatten(self.current, " ") + " ] " + target
             )
 
-
             zip_copy = False
             if self._zip and self._ncommands == 1:
                 zip_copy = True
@@ -625,7 +653,48 @@ def run_this(os_command, self, output="one", out_file=None):
 
             os_command = tidy_command(os_command)
 
-            target = run_cdo(os_command, target, out_file)
+            if "mergetime" in os_command:
+                try:
+                    target = run_cdo(os_command, target, out_file, precision = self._precision)
+                except:
+
+                    var_list = []
+                    var_com = []
+
+                    for ff in self:
+                        var_list += nc_variables(ff)
+                        var_com.append(nc_variables(ff))
+                    new_list = []
+
+                    for var in set(var_list):
+                        if len(var_com) == len([x for x in var_com if var in x]):
+                            new_list.append(var)
+                    if (
+                        "1.9.4" in cdo_version()
+                        or "1.9.5" in cdo_version()
+                        or "1.9.6" in cdo_version()
+                        or "1.9.7" in cdo_version()
+                        or "1.9.8" in cdo_version()
+                    ):
+
+                        target = run_cdo(os_command, target, out_file, precision = self._precision)
+                    else:
+                        f_list = ",".join(new_list)
+                        os_command = os_command.replace(
+                            "-mergetime ", f'-mergetime -apply,"-selname,{f_list}" '
+                        )
+
+                        removed = ",".join(
+                            [x for x in set(var_list) if x not in new_list]
+                        )
+                        if len([x for x in set(var_list) if x not in new_list]) > 0:
+                            warnings.warn(
+                                f"The following variables are not in all files, so were ignored when merging: {removed}"
+                            )
+
+                        target = run_cdo(os_command, target, out_file, precision = self._precision)
+            else:
+                target = run_cdo(os_command, target, out_file, precision = self._precision)
 
             remove_safe(target)
 
