@@ -94,6 +94,44 @@ def tsum(self, over="time"):
     time_stat(self, stat="sum", over=over)
 
 
+def na_count(self, over="time"):
+    """
+    Calculate the number of missing values
+
+    Parameters
+    -------------
+    over: str or list
+        Time periods to to the count over over. Options are 'time', 'year', 'month', 'day'.
+
+    """
+
+    self.run()
+
+    for vv in self.variables:
+        self.cdo_command(f"-aexpr,'{vv}=isMissval({vv})'")
+
+    self.tsum(over=over)
+
+
+def na_frac(self, over="time"):
+    """
+    Calculate the number of missing values
+
+    Parameters
+    -------------
+    over: str or list
+        Time periods to to the count over over. Options are 'time', 'year', 'month', 'day'.
+
+    """
+
+    self.run()
+
+    for vv in self.variables:
+        self.cdo_command(f"-aexpr,'{vv}=isMissval({vv})'")
+
+    self.tmean(over=over)
+
+
 def tmean(self, over="time"):
     """
     Calculate the temporal mean of all variables
@@ -334,7 +372,9 @@ def tvariance(self, over="time"):
 
         >>> ds.tvariance( "day")
     """
-    warnings.warn(message="tvariance is deprecated and has been renamed tvar. Please use tvar!")
+    warnings.warn(
+        message="tvariance is deprecated and has been renamed tvar. Please use tvar!"
+    )
     time_stat(self, stat="var", over=over)
 
 
@@ -378,7 +418,6 @@ def tvar(self, over="time"):
     time_stat(self, stat="var", over=over)
 
 
-
 def tstdev(self, over="time"):
     """
     Calculate the temporal standard deviation of all variables
@@ -550,7 +589,7 @@ def tpercentile(self, p=None, over="time"):
         )
 
         cdo_command = tidy_command(cdo_command)
-        target = run_cdo(cdo_command, target)
+        target = run_cdo(cdo_command, target, precision=self._precision)
         new_files.append(target)
         new_commands.append(cdo_command)
 
